@@ -1,7 +1,7 @@
 /***************************************************************************
  *   Copyright (C) 2004-2011 by Pere Constans
  *   constans@molspaces.com
- *   cb2Bib version 1.4.6. Licensed under the GNU GPL version 3.
+ *   cb2Bib version 1.4.7. Licensed under the GNU GPL version 3.
  *   See the LICENSE file that comes with this distribution.
  ***************************************************************************/
 #ifndef BIBPARSER_H
@@ -51,6 +51,10 @@ public:
     {
         return _journal_dbP->retrieve(name);
     }
+    inline const QStringList& abbreviatedSimplifiedJournalList() const
+    {
+        return _journal_dbP->abbreviatedSimplifiedList();
+    }
     inline QString fullJournal(const QString& name) const
     {
         return _journal_dbP->retrieveFull(name);
