@@ -1,13 +1,12 @@
 /***************************************************************************
  *   Copyright (C) 2004-2011 by Pere Constans
  *   constans@molspaces.com
- *   cb2Bib version 1.4.6. Licensed under the GNU GPL version 3.
+ *   cb2Bib version 1.4.7. Licensed under the GNU GPL version 3.
  *   See the LICENSE file that comes with this distribution.
  ***************************************************************************/
 #include "bibSearcher.h"
 
 #include "bibParser.h"
-#include "cb2bib_utilities.h"
 #include "settings.h"
 
 #include <QCoreApplication>
@@ -117,6 +116,11 @@ bibSearcher::bibSearcher(bibParser* bp, const QString& bib_dir, QObject* parento
         _do_search_similar_citeid = '@';
 }
 
+bibSearcher::bibSearcher() : _bpP(0), _do_search_similar(false)
+{
+    clear();
+}
+
 
 void bibSearcher::addPattern(bool Not, bool caseSensitive, const QString& patternType, const QString& scope,
                              const QChar& yearScope, const QString& pattern)
@@ -150,18 +154,12 @@ void bibSearcher::exec()
     }
 
     // Search In Files
-    QFileInfoList flist;
-    if (_all_bibtex_files)
-    {
-        QDir bibdir(_bibtex_dir);
-        flist = bibdir.entryInfoList(QStringList() << "*.bib");
-    }
-    else
-        flist.append(QFileInfo(_bibtex_file));
-
+    const QStringList flist(_all_bibtex_files ?
+                            c2bUtils::filesInDir(_bibtex_dir, QStringList() << "*.bib") :
+                            c2bUtils::filesInDir(_bibtex_file, QStringList() << "*.bib"));
     for (int i = 0; i < flist.count(); ++i)
     {
-        search(flist.at(i).absoluteFilePath());
+        search(flist.at(i));
         if (_aborted)
         {
             clear();
@@ -181,6 +179,31 @@ void bibSearcher::exec()
     _log_string += tr("% Total Unique Hits: %1\n").arg(_hits_map.count());
 }
 
+QString bibSearcher::searchDocumentKeyword(const QString& bibtexfn, const QString& documentfn, const QString& keyword)
+{
+    bibSearcher bs;
+    QString exc;
+    bs._documents.load(bibtexfn, documentContents::Raw);
+    if (bs._documents.setCurrent(documentfn))
+    {
+        QString p(keyword);
+        p.replace(QRegExp("\\W"), ".{0,5}");
+        p.replace("s", ".?");
+        p = "\\b" + p + "\\w*\\b";
+        bs.addPattern(false, false, searchPattern::type(searchPattern::RegularExpression), "all", QChar(), p);
+        QString document(bs._documents.current().text());
+        c2bUtils::stripDiacritics(document);
+        if (bs._patterns.at(0).matches(document))
+        {
+            exc = c2bUtils::fileToString(":/htm/htm/excerpts.html");
+            exc.replace("GET_EXCERPTS_TITLE", keyword);
+            exc.replace("GET_EXCERPTS", bs.excerpts(document).mid(20));
+        }
+    }
+    bs._documents.unload();
+    return exc;
+}
+
 void bibSearcher::abort()
 {
     _aborted = true;
@@ -356,8 +379,8 @@ const QString bibSearcher::excerpts(const QString& contents) const
 
     for (int i = 0; i < _patterns.count(); ++i)
     {
-        const searchPattern& pattern = _patterns.at(i);
-        const searchPattern::modifiers& modifier = pattern.modifier();
+        const searchPattern& pattern(_patterns.at(i));
+        const searchPattern::modifiers& modifier(pattern.modifier());
         if (modifier.NOT)
             continue;
         if (modifier.scope != "all" && modifier.scope != "file")
