@@ -1,7 +1,7 @@
 /***************************************************************************
  *   Copyright (C) 2004-2011 by Pere Constans
  *   constans@molspaces.com
- *   cb2Bib version 1.4.6. Licensed under the GNU GPL version 3.
+ *   cb2Bib version 1.4.7. Licensed under the GNU GPL version 3.
  *   See the LICENSE file that comes with this distribution.
  ***************************************************************************/
 #ifndef BIBSEARCHER_H
@@ -29,6 +29,8 @@ public:
     bibSearcher(bibParser* bp, const QString& bib_dir, QObject* parento = 0);
     inline ~bibSearcher() {}
 
+    static QString searchDocumentKeyword(const QString& bibtexfn, const QString& documentfn, const QString& keyword);
+
     const QString highlight(const QString& abstract) const;
     void addPattern(bool Not, bool caseSensitive, const QString& patternType, const QString& scope,
                     const QChar& yearScope, const QString& pattern);
@@ -94,6 +96,8 @@ public slots:
 
 
 private:
+    bibSearcher();
+
     QList<searchPattern> _patterns;
     QMap<QString, QString> _hits_map;
     QString _bibtex_dir;
