@@ -1,7 +1,7 @@
 /***************************************************************************
  *   Copyright (C) 2004-2011 by Pere Constans
  *   constans@molspaces.com
- *   cb2Bib version 1.4.6. Licensed under the GNU GPL version 3.
+ *   cb2Bib version 1.4.7. Licensed under the GNU GPL version 3.
  *   See the LICENSE file that comes with this distribution.
  ***************************************************************************/
 #include "cb2bib_utilities.h"
@@ -56,6 +56,57 @@ QString& simplifyString(QString& str)
     return str;
 }
 
+QString& fillString(QString& str, const QStringMatcher& pattern, const QChar& ch)
+{
+    if (str.length() == 0)
+        return str;
+    const int pl(pattern.pattern().length());
+    const ushort uch(ch.unicode());
+    ushort* const c0((ushort*)str.data());
+    int p(0);
+    while (p >= 0)
+    {
+        p = pattern.indexIn(str, p);
+        if (p > -1)
+        {
+            ushort* c(c0 + p);
+            const ushort* const cpl(c + pl);
+            while (c < cpl)
+                *c++ = uch;
+            p += pl;
+        }
+    }
+    return str;
+}
+
+QString& fillString(QString& str, const QString& pattern, const QChar& ch)
+{
+    return fillString(str, QStringMatcher(pattern, Qt::CaseSensitive), ch);
+}
+
+QString& fillString(QString& str, const QRegExp& pattern, const QChar& ch)
+{
+    if (str.length() == 0)
+        return str;
+    const ushort uch(ch.unicode());
+    ushort* const c0((ushort*)str.data());
+    int p(0);
+    while (p >= 0)
+    {
+        p = pattern.indexIn(str, p);
+        if (p > -1)
+        {
+            const int pl(pattern.matchedLength());
+            ushort* c(c0 + p);
+            const ushort* const cpl(c + pl);
+            while (c < cpl)
+                *c++ = uch;
+            p += pl;
+        }
+    }
+    return str;
+}
+
 static const unsigned short _cyrillic_to_ascii[] =
 {
     // Code points 1024 to 1309
@@ -231,6 +282,12 @@ QString toAscii(const QString& str, const AsciiConversion type)
     return ascii;
 }
 
+QString& stripDiacritics(QString& str)
+{
+    _to_ascii_transliterate(str);
+    return str;
+}
+
 QString& c2bToBib(QString& str)
 {
     // Escape common Extended Latin Characters
