@@ -1,7 +1,7 @@
 /***************************************************************************
  *   Copyright (C) 2004-2011 by Pere Constans
  *   constans@molspaces.com
- *   cb2Bib version 1.4.6. Licensed under the GNU GPL version 3.
+ *   cb2Bib version 1.4.7. Licensed under the GNU GPL version 3.
  *   See the LICENSE file that comes with this distribution.
  ***************************************************************************/
 #ifndef CB2BIB_UTILITIES_H
@@ -28,7 +28,11 @@ extern QString setCapitalization(const QString& str);
 extern QString toAscii(const QString& str, const AsciiConversion type);
 extern QString& bibToC2b(QString& str);
 extern QString& c2bToBib(QString& str);
+extern QString& fillString(QString& str, const QRegExp& pattern, const QChar& ch);
+extern QString& fillString(QString& str, const QString& pattern, const QChar& ch);
+extern QString& fillString(QString& str, const QStringMatcher& pattern, const QChar& ch);
 extern QString& simplifyString(QString& str);
+extern QString& stripDiacritics(QString& str);
 
 static const QRegExp pnasGreekLetters("[\\{\\[\\(](alpha|beta|gamma|delta|varepsilon|"
                                       "zeta|eta|theta|iota|kappa|lambda|mu|nu|xi|"
@@ -270,6 +274,24 @@ inline bool stringToFile(const QString& str, const QString& fn)
         return false;
 }
 
+inline QStringList filesInDir(const QString& dir, const QStringList& filters)
+{
+    QStringList files;
+    QFileInfo fi(dir);
+    if (!fi.exists())
+        return files;
+    if (fi.isDir())
+    {
+        QDir d(dir);
+        const QFileInfoList dil(d.entryInfoList(filters));
+        for (int i = 0; i < dil.count(); ++i)
+            files.append(QDir::toNativeSeparators(QDir::cleanPath(dil.at(i).absoluteFilePath())));
+    }
+    else if (fi.isFile())
+        files.append(QDir::toNativeSeparators(QDir::cleanPath(fi.absoluteFilePath())));
+    return files;
+}
+
 inline int nearInteger(double a)
 {
     int ia = int(a);
