@@ -1,7 +1,7 @@
 /***************************************************************************
  *   Copyright (C) 2004-2011 by Pere Constans
  *   constans@molspaces.com
- *   cb2Bib version 1.4.6. Licensed under the GNU GPL version 3.
+ *   cb2Bib version 1.4.7. Licensed under the GNU GPL version 3.
  *   See the LICENSE file that comes with this distribution.
  ***************************************************************************/
 #ifndef JOURNALDB_H
@@ -50,6 +50,10 @@ public:
     {
         return JExtended_simp.at(index);
     }
+    inline const QStringList& abbreviatedSimplifiedList() const
+    {
+        return JAbbrev_simp_w;
+    }
 
 
 private:
