@@ -1,7 +1,7 @@
 /***************************************************************************
  *   Copyright (C) 2004-2011 by Pere Constans
  *   constans@molspaces.com
- *   cb2Bib version 1.4.6. Licensed under the GNU GPL version 3.
+ *   cb2Bib version 1.4.7. Licensed under the GNU GPL version 3.
  *   See the LICENSE file that comes with this distribution.
  ***************************************************************************/
 #include "settings.h"
@@ -17,13 +17,18 @@
 */
 settings::settings(const QString& organization, const QString& application, QObject* parento)
     : QSettings(organization, application, parento)
-{}
+{
+    _init();
+}
 
 settings::settings(const QString& fn, QObject* parento) : QSettings(fn, QSettings::IniFormat, parento)
-{}
+{
+    _init();
+}
 
 settings::settings(QObject* parento) : QSettings(C2B_ORGANIZATION, C2B_APPLICATION, parento)
 {
+    _init();
     init(false);
 }
 
@@ -35,25 +40,6 @@ settings::~settings()
 
 void settings::init(bool command_line)
 {
-    _c2bApp = QCoreApplication::instance();
-    Q_ASSERT_X(_c2bApp, "settings", "application was not instantiated");
-    Q_ASSERT_X(!_c2b_global_settings, "settings", "settings already instantiated");
-    settings::_c2b_global_settings = this;
-    _application_pid = QString::number(_c2bApp->applicationPid());
-
-    _is_loaded = true;
-    cl_bibedit = false;
-    cl_cb2bib_import_mode = false;
-    cl_cb2bib_tests = false;
-    cl_citer = false;
-    cl_configure = false;
-    cl_doc2bib = false;
-    cl_html_annote = false;
-    cl_import = false;
-    cl_sloppy = false;
-    cl_txt2bib = false;
-    cl_user_configured = false;
-    cl_view_annote = false;
     if (command_line)
         readCommandLine();
     if (cl_user_configured)
@@ -75,6 +61,30 @@ void settings::init(bool command_line)
         setValue("cb2Bib/RelativePdfDirectory", true);
 }
 
+void settings::_init()
+{
+    _c2bApp = QCoreApplication::instance();
+    Q_ASSERT_X(_c2bApp, "settings", "application was not instantiated");
+    Q_ASSERT_X(!_c2b_global_settings, "settings", "settings already instantiated");
+    settings::_c2b_global_settings = this;
+    _application_pid = QString::number(_c2bApp->applicationPid());
+
+    _is_loaded = true;
+    cl_bibedit = false;
+    cl_cb2bib_import_mode = false;
+    cl_cb2bib_tests = false;
+    cl_citer = false;
+    cl_configure = false;
+    cl_doc2bib = false;
+    cl_html_annote = false;
+    cl_import = false;
+    cl_index = false;
+    cl_sloppy = false;
+    cl_txt2bib = false;
+    cl_user_configured = false;
+    cl_view_annote = false;
+}
+
 void settings::setDefaultValue(const QString& key, const QVariant& defaulValue)
 {
     // Makes sure a key is initialized
@@ -100,6 +110,7 @@ void settings::setDefaults()
     defaults["cb2Bib/ExifToolBin"] = QDir::cleanPath(C2B_METADATAPARSER_EXIFTOOL_BIN);
     defaults["cb2Bib/JournalFile"] = QDir::cleanPath(C2B_DATA_DIR + dsep + "data/abbreviations.txt");
     defaults["cb2Bib/NetworkFile"] = QDir::cleanPath(C2B_DATA_DIR + dsep + "data/netqinf.txt");
+    defaults["cb2Bib/PosLexiconFile"] = QDir::cleanPath(C2B_DATA_DIR + dsep + "data/lexicon.pos");
     defaults["cb2Bib/PostprocessBibTeXBin"] = QDir::cleanPath(C2B_DATA_DIR + dsep + C2B_POSTPROCESS_BIBTEX_BIN);
     defaults["cb2Bib/PreparserBin"] = QDir::cleanPath(C2B_DATA_DIR + dsep + "c2btools/ris2bib");
     defaults["cb2Bib/RegularExpressionFile"] = QDir::cleanPath(C2B_DATA_DIR + dsep + "data/regexps.txt");
@@ -118,6 +129,7 @@ void settings::setDefaults()
     defaults["cb2Bib/ExifToolBin"] = QDir::cleanPath(C2B_METADATAPARSER_EXIFTOOL_BIN);
     defaults["cb2Bib/JournalFile"] = QDir::cleanPath(C2B_DATA_DIR_MAC + "/data/abbreviations.txt");
     defaults["cb2Bib/NetworkFile"] = QDir::cleanPath(C2B_DATA_DIR_MAC + "/data/netqinf.txt");
+    defaults["cb2Bib/PosLexiconFile"] = QDir::cleanPath(C2B_DATA_DIR_MAC + dsep + "data/lexicon.pos");
     defaults["cb2Bib/PostprocessBibTeXBin"] = QDir::cleanPath(C2B_DATA_DIR_MAC + C2B_POSTPROCESS_BIBTEX_BIN);
     defaults["cb2Bib/PreparserBin"] = QDir::cleanPath(C2B_DATA_DIR_MAC + "/c2btools/ris2bib");
     defaults["cb2Bib/RegularExpressionFile"] = QDir::cleanPath(C2B_DATA_DIR_MAC + "/data/regexps.txt");
@@ -131,6 +143,7 @@ void settings::setDefaults()
     defaults["cb2Bib/BrowserCssFile"] = C2B_APP_DIR + "data\\references.css";
     defaults["cb2Bib/JournalFile"] = C2B_APP_DIR + "data\\abbreviations.txt";
     defaults["cb2Bib/NetworkFile"] = C2B_APP_DIR + "data\\netqinf.txt";
+    defaults["cb2Bib/PosLexiconFile"] = C2B_APP_DIR + "data\\lexicon.pos";
     defaults["cb2Bib/PostprocessBibTeXBin"] = C2B_APP_DIR + C2B_POSTPROCESS_BIBTEX_BIN;
     defaults["cb2Bib/PreparserBin"] = C2B_APP_DIR + "c2btools\\ris2bib.bat";
     defaults["cb2Bib/RegularExpressionFile"] = C2B_APP_DIR + "data\\regexps.txt";
@@ -213,6 +226,8 @@ settings* settings::_c2b_global_settings = 0;
         --doc2bib fn1.doc [fn2.doc ... ] reference.bib     Extract reference from document file
         --txt2bib fn1.txt [fn2.txt ... ] reference.bib     Extract reference from text file
 
+        --index [bibdirname]                               Extract keywords from document files
+
         --html-annote filename.tex                         Convert annote file to HTML
         --view-annote filename.tex                         Convert and visualize annote file
         --view-annote filename.tex.html                    Visualize annote file
@@ -261,6 +276,7 @@ bool settings::isConsoleMode(int& argc, char** argv)
             strcmp(argv[i], "--html-annote") == 0 ||
             strcmp(argv[i], "--doc2bib") == 0 ||
             strcmp(argv[i], "--txt2bib") == 0 ||
+            strcmp(argv[i], "--index") == 0 ||
             strcmp(argv[i], "--test") == 0)
             return true;
     return false;
@@ -340,6 +356,10 @@ void settings::readCommandLine()
             if (arguments.at(i) == "--conf")
             {
                 checkCommand(&cl_user_configured, &actions);
+                _conf_command_flags.append("--conf");
+                const int j(i + 1);
+                if (j < arguments.count() && !arguments.at(j).startsWith('-'))
+                    _conf_command_flags.append(arguments.at(j));
                 --actions;
             }
             else if (arguments.at(i) == "--sloppy")
@@ -364,6 +384,8 @@ void settings::readCommandLine()
                 checkCommand(&cl_doc2bib, &actions);
             else if (arguments.at(i) == "--txt2bib")
                 checkCommand(&cl_txt2bib, &actions);
+            else if (arguments.at(i) == "--index")
+                checkCommand(&cl_index, &actions);
             else if (arguments.at(i).startsWith("--test"))
                 cl_cb2bib_tests = true;
             else
@@ -375,12 +397,14 @@ void settings::readCommandLine()
         _is_loaded = false;
     if (cl_sloppy && !(cl_doc2bib || cl_txt2bib))
         _is_loaded = false;
+    if (cl_index && (cl_bibedit || cl_citer || cl_import || cl_html_annote || cl_view_annote || cl_doc2bib || cl_txt2bib))
+        _is_loaded = false;
     if (!_is_loaded)
     {
         printUsage();
         return;
     }
-    cl_console_mode = cl_html_annote || cl_cb2bib_tests || cl_doc2bib || cl_txt2bib;
+    cl_console_mode = cl_html_annote || cl_cb2bib_tests || cl_doc2bib || cl_txt2bib || cl_index;
     cl_cb2bib_mode = !(cl_console_mode || cl_configure || cl_view_annote);
 
     // Check for cl_user_configured
@@ -424,6 +448,18 @@ void settings::readCommandLine()
                 break;
             }
     }
+    // Check for --index BibTeX dirname
+    else if (cl_index)
+    {
+        for (int i = 1; i < arguments.count(); ++i)
+            if (arguments.at(i) == "--index")
+            {
+                const int j(i + 1);
+                if (j < arguments.count() && !arguments.at(j).startsWith('-'))
+                    cl_index_dirname.append(decodeFilename(arguments.at(j)));
+                break;
+            }
+    }
     // Check for annote file
     else if (cl_html_annote || cl_view_annote)
     {
@@ -509,6 +545,8 @@ void settings::printUsage()
     fprintf(stdout, "       --doc2bib fn1.doc [fn2.doc ... ] reference.bib     Extract reference from document file\n");
     fprintf(stdout, "       --txt2bib fn1.txt [fn2.txt ... ] reference.bib     Extract reference from text file\n");
     fprintf(stdout, "\n");
+    fprintf(stdout, "       --index [bibdirname]                               Extract keywords from document files\n");
+    fprintf(stdout, "\n");
     fprintf(stdout, "       --html-annote filename.tex                         Convert annote file to HTML\n");
     fprintf(stdout, "       --view-annote filename.tex                         Convert and visualize annote file\n");
     fprintf(stdout, "       --view-annote filename.tex.html                    Visualize annote file\n");
