@@ -1,7 +1,7 @@
 /***************************************************************************
  *   Copyright (C) 2004-2011 by Pere Constans
  *   constans@molspaces.com
- *   cb2Bib version 1.4.6. Licensed under the GNU GPL version 3.
+ *   cb2Bib version 1.4.7. Licensed under the GNU GPL version 3.
  *   See the LICENSE file that comes with this distribution.
  ***************************************************************************/
 #ifndef SETTINGS_H
@@ -29,6 +29,7 @@ public:
     QString cl_annote_filename;
     QString cl_extract_output_filename;
     QString cl_import_filename;
+    QString cl_index_dirname;
     QStringList cl_bibedit_filenames;
     QStringList cl_citer_filenames;
     QStringList cl_extract_input_filenames;
@@ -42,6 +43,7 @@ public:
     bool cl_doc2bib;
     bool cl_html_annote;
     bool cl_import;
+    bool cl_index;
     bool cl_sloppy;
     bool cl_txt2bib;
     bool cl_user_configured;
@@ -91,6 +93,10 @@ public:
     {
         return _is_usb_configured_mode;
     }
+    inline QStringList confCommandFlags() const
+    {
+        return _conf_command_flags;
+    }
     inline QString tempPath() const
     {
         return _tmp_dir_path;
@@ -126,9 +132,11 @@ private:
     QDir _c2bAppDir;
     QString _application_pid;
     QString _tmp_dir_path;
+    QStringList _conf_command_flags;
     bool _is_loaded;
     bool _is_usb_configured_mode;
     static settings* _c2b_global_settings;
+    void _init();
     void checkCommand(bool* action, int* actions);
     void printUsage();
     void readCommandLine();
