@@ -1,7 +1,7 @@
 /***************************************************************************
  *   Copyright (C) 2004-2011 by Pere Constans
  *   constans@molspaces.com
- *   cb2Bib version 1.4.6. Licensed under the GNU GPL version 3.
+ *   cb2Bib version 1.4.7. Licensed under the GNU GPL version 3.
  *   See the LICENSE file that comes with this distribution.
  ***************************************************************************/
 #include "texToHtml.h"
@@ -264,8 +264,7 @@ void texToHtml::retrieveReferences()
 {
     if (_bibtex_directory.isEmpty())
         _bibtex_directory = QFileInfo(_settingsP->fileName("cb2Bib/BibTeXFile")).path();
-    QDir bibdir(_bibtex_directory);
-    QFileInfoList flist = bibdir.entryInfoList(QStringList() << "*.bib");
+    const QStringList flist(c2bUtils::filesInDir(_bibtex_directory, QStringList() << "*.bib"));
     const QStringList keys(_cites.keys());
     QStringList fields;
     fields.append("author");
@@ -285,7 +284,7 @@ void texToHtml::retrieveReferences()
     for (int i = 0; i < flist.count(); ++i)
     {
         if (done) return;
-        const QString bib_contents(c2bUtils::fileToString(flist.at(i).absoluteFilePath()));
+        const QString bib_contents(c2bUtils::fileToString(flist.at(i)));
         done = true;
         for (int j = 0; j < keys.count(); ++j)
         {
