@@ -1,15 +1,20 @@
 /***************************************************************************
  *   Copyright (C) 2004-2011 by Pere Constans
  *   constans@molspaces.com
- *   cb2Bib version 1.4.6. Licensed under the GNU GPL version 3.
+ *   cb2Bib version 1.4.7. Licensed under the GNU GPL version 3.
  *   See the LICENSE file that comes with this distribution.
  ***************************************************************************/
 #ifndef C2BCITERMODEL_H
 #define C2BCITERMODEL_H
 
+#include "c2bCiter.h"
+#include "c2bCiterHistory.h"
+
+#include <collectionAnalyzer.h>
 
 #include <QAbstractTableModel>
 #include <QColor>
+#include <QHash>
 #include <QStringList>
 #include <QVector>
 
@@ -21,28 +26,27 @@ class c2bCiterModel : public QAbstractTableModel
 
 public:
     c2bCiterModel(QObject* parento = 0);
-    ~c2bCiterModel();
-
-    enum Format {AJYT, IT, JYA, T, YAJT};
-
-    struct State
-    {
-        inline State() : format(AJYT) {}
-        Format format;
-        QString citation;
-    };
+    inline ~c2bCiterModel() {}
 
     QList<int> sizeHintForColumns() const;
     QStringList dataSelectedCiteIds() const;
     QVariant data(const QModelIndex& i, int role = Qt::DisplayRole) const;
-    State currentState(const QModelIndex& current_index) const;
+    c2bCiter::State currentState(const QModelIndex& current_index) const;
+    void clearCurrentFilter(QModelIndex* current_index);
     void clearSelection();
     void loadCitations(const QString& fn);
     void loadCitations(const QStringList& fns);
-    void reloadCitations(const QStringList& fns, const State& state, QModelIndex* current_index);
-    void setFilter(const QString& pattern, QModelIndex* current_index);
+    void reloadCitations(const QStringList& fns, const c2bCiter::State& state, QModelIndex* current_index);
+    void restoreNextState(QModelIndex* current_index);
+    void restorePreviousState(QModelIndex* current_index);
+    void restoreState(const c2bCiter::State& s, QModelIndex* current_index);
+    void setDocumentFilter(const QString& documentfn, QModelIndex* current_index);
+    void setGlossaryView(QModelIndex* current_index);
+    void setKeywordFilter(const QString& keyword, QModelIndex* current_index);
+    void setPatternFilter(const QString& pattern, QModelIndex* current_index, const c2bCiter::Format format);
+    void setRelatedFilter(const QString& documentfn, QModelIndex* current_index, const c2bCiter::Format format = c2bCiter::R);
     void setSelectedFilter(QModelIndex* current_index);
-    void updateFormat(const Format format, QModelIndex* current_index);
+    void updateFormat(const c2bCiter::Format format, QModelIndex* current_index);
 
     inline QString dataBibTeXPosition(const QModelIndex& i) const
     {
@@ -58,7 +62,7 @@ public:
     inline QString dataFile(const QModelIndex& i) const
     {
         if (_is_index_valid(i))
-            return _file.at(_offset(i.row()));
+            return _file.key(_offset(i.row()));
         else
             return QString();
     }
@@ -69,6 +73,35 @@ public:
         else
             return QString();
     }
+    inline QString dataKeyword(const QModelIndex& i) const
+    {
+        if (_format == c2bCiter::K && _is_index_valid(i))
+            return _analyzer.sentence(_offset(i.row()));
+        else
+            return QString();
+    }
+    inline c2bCiter::KeywordData keywordData(const QModelIndex& i) const
+    {
+        c2bCiter::KeywordData kd;
+        if (_filter == c2bCiter::Document && _format == c2bCiter::K)
+        {
+            kd.bibtexfn = QString(_bibtex_position.at(_file.value(_fstring))).remove(QRegExp(":(\\d+)$"));
+            kd.documentfn = _fstring;
+            kd.keyword = dataKeyword(i);
+        }
+        else if (_filter == c2bCiter::Keyword)
+        {
+            kd.bibtexfn = dataBibTeXFile(i);
+            kd.documentfn = dataFile(i);
+            kd.keyword = _fstring;
+        }
+        kd.valid = !kd.bibtexfn.isEmpty() && !kd.documentfn.isEmpty() && !kd.keyword.isEmpty();
+        return kd;
+    }
+    inline bool isKeywordView() const
+    {
+        return _format == c2bCiter::K;
+    }
     inline Qt::ItemFlags flags(const QModelIndex& /* i */) const
     {
         return Qt::ItemIsSelectable | Qt::ItemIsEnabled;
@@ -85,6 +118,26 @@ public:
     {
         return _column_count;
     }
+    inline c2bCiter::Filter currentFilter() const
+    {
+        return _filter;
+    }
+    inline c2bCiter::Format currentFormat() const
+    {
+        return _format;
+    }
+    inline c2bCiter::Format lastCitationFormat() const
+    {
+        return _history.lastCitationFormat();
+    }
+    inline void restoreLastNonKeywordState(QModelIndex* current_index)
+    {
+        restoreState(_history.lastNonKeywordState(), current_index);
+    }
+    inline void restoreLastNonSelectedState(QModelIndex* current_index)
+    {
+        restoreState(_history.lastNonSelectedState(), current_index);
+    }
 
 
 public slots:
@@ -92,7 +145,9 @@ public slots:
 
 
 signals:
-    void statusMessage(const QString& ms);
+    void patternFilterChanged(const QString& pattern);
+    void statusLabel(const QString& label);
+    void statusMessage(const QString& message);
 
 
 private:
@@ -102,35 +157,50 @@ private:
     QString _display_ajyt(const int row, const int column) const;
     QString _display_it(const int row, const int column) const;
     QString _display_jya(const int row, const int column) const;
+    QString _display_k(const int row, const int column) const;
+    QString _display_r(const int row, const int column) const;
     QString _display_t(const int row, const int column) const;
     QString _display_yajt(const int row, const int column) const;
     QString(c2bCiterModel::*_display_ptr)(const int row, const int column) const;
+    int _current_item(const QString& index_data) const;
     int _offset(const int i) const;
     void _add_citations(const QString& fn);
     void _clear();
     void _set_mapping();
     void _set_sort_indices();
     void _set_table_data();
-    void _update_current_index(const int current_citation, QModelIndex* current_index);
-    void _update_format(const Format format = AJYT, QModelIndex* current_index = 0);
+    void _update_current_index(const int current_item, QModelIndex* current_index) const;
+    void _update_format(const c2bCiter::Format format = c2bCiter::AJYT);
 
     inline bool _is_index_valid(const QModelIndex& i) const
     {
         const int r(i.row());
         return _row_count > 0 && r >= 0 && r < _row_count;
     }
-    inline int _current_citation(const QModelIndex& current_index) const
+    inline int _current_item(const QModelIndex& current_index) const
     {
-        return _is_index_valid(current_index) ? _offset(current_index.row()) : _previous_citation;
+        return _is_index_valid(current_index) ? _offset(current_index.row()) : -1;
+    }
+    inline int _current_item(const QModelIndex* const current_index) const
+    {
+        return current_index ? _current_item(*current_index) : -1;
     }
 
+    // Collection analyzer helper
+    collectionAnalyzer _analyzer;
+
+    // History helper
+    QString _fstring;
+    c2bCiter::Filter _filter;
+    c2bCiter::Format _format;
+    c2bCiterHistory _history;
+
     // Model data
-    Format _format;
+    QHash<QString, int> _file;
     QList<uint> _included_date;
     QStringList _author_string;
     QStringList _bibtex_position;
     QStringList _citeId;
-    QStringList _file;
     QStringList _journal;
     QStringList _title;
     QStringList _url;
@@ -147,13 +217,13 @@ private:
     QVector<int> _map_t;
     QVector<int> _map_yajt;
     QVector<int>* _mapping;
-    bool _is_filter_set;
     int _author_count;
     int _author_max_length;
     int _citation_count;
     int _column_count;
     int _journal_max_length;
-    int _previous_citation;
+    int _keyword_count;
+    int _related_count;
     int _row_count;
 
 };
